/**
 * Copyright (c) 2022, Salesforce, Inc.,
 * All rights reserved.
 * For full license text, see the LICENSE.txt file
 */

/*
 *  ATTENTION!
 *  THIS IS A GENERATED FILE FROM https://github.com/salesforce-experience-platform-emu/lds-lightning-platform
 *  If you would like to contribute to LDS, please follow the steps outlined in the git repo.
 *  Any changes made to this file in p4 will be automatically overwritten.
 *  *******************************************************************************************
 */
/* proxy-compat-disable */
import { createInstrumentedAdapter, createLDSAdapter, createWireAdapterConstructor, createImperativeAdapter } from 'force/ldsBindings';
import { withDefaultLuvio } from 'force/ldsEngine';
import { serializeStructuredKey, ingestShape, deepFreeze, StoreKeyMap, createResourceParams as createResourceParams$2, typeCheckConfig as typeCheckConfig$2, buildNetworkSnapshotCachePolicy as buildNetworkSnapshotCachePolicy$1 } from 'force/luvioEngine';

const { hasOwnProperty: ObjectPrototypeHasOwnProperty } = Object.prototype;
const { keys: ObjectKeys$1, create: ObjectCreate$1 } = Object;
const { isArray: ArrayIsArray$1 } = Array;
/**
 * Validates an adapter config is well-formed.
 * @param config The config to validate.
 * @param adapter The adapter validation configuration.
 * @param oneOf The keys the config must contain at least one of.
 * @throws A TypeError if config doesn't satisfy the adapter's config validation.
 */
function validateConfig(config, adapter, oneOf) {
    const { displayName } = adapter;
    const { required, optional, unsupported } = adapter.parameters;
    if (config === undefined ||
        required.every(req => ObjectPrototypeHasOwnProperty.call(config, req)) === false) {
        throw new TypeError(`adapter ${displayName} configuration must specify ${required.sort().join(', ')}`);
    }
    if (oneOf && oneOf.some(req => ObjectPrototypeHasOwnProperty.call(config, req)) === false) {
        throw new TypeError(`adapter ${displayName} configuration must specify one of ${oneOf.sort().join(', ')}`);
    }
    if (unsupported !== undefined &&
        unsupported.some(req => ObjectPrototypeHasOwnProperty.call(config, req))) {
        throw new TypeError(`adapter ${displayName} does not yet support ${unsupported.sort().join(', ')}`);
    }
    const supported = required.concat(optional);
    if (ObjectKeys$1(config).some(key => !supported.includes(key))) {
        throw new TypeError(`adapter ${displayName} configuration supports only ${supported.sort().join(', ')}`);
    }
}
function untrustedIsObject(untrusted) {
    return typeof untrusted === 'object' && untrusted !== null && ArrayIsArray$1(untrusted) === false;
}
function areRequiredParametersPresent(config, configPropertyNames) {
    return configPropertyNames.parameters.required.every(req => req in config);
}
const snapshotRefreshOptions = {
    overrides: {
        headers: {
            'Cache-Control': 'no-cache',
        },
    }
};
function generateParamConfigMetadata(name, required, resourceType, typeCheckShape, isArrayShape = false, coerceFn) {
    return {
        name,
        required,
        resourceType,
        typeCheckShape,
        isArrayShape,
        coerceFn,
    };
}
function buildAdapterValidationConfig(displayName, paramsMeta) {
    const required = paramsMeta.filter(p => p.required).map(p => p.name);
    const optional = paramsMeta.filter(p => !p.required).map(p => p.name);
    return {
        displayName,
        parameters: {
            required,
            optional,
        }
    };
}
const keyPrefix = 'cdp-query';

const { keys: ObjectKeys, create: ObjectCreate, assign: ObjectAssign } = Object;
const { isArray: ArrayIsArray } = Array;
const { stringify: JSONStringify } = JSON;
function createLink(ref) {
    return {
        __ref: serializeStructuredKey(ref),
    };
}

const TTL = 100;
const VERSION$1 = "318ac9ee34fe90764a782a1b3eac7d16";
function validate$1(obj, path = 'CdpQueryOutputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_data = obj.data;
        const path_data = path + '.data';
        if (!ArrayIsArray(obj_data)) {
            return new TypeError('Expected "array" but received "' + typeof obj_data + '" (at "' + path_data + '")');
        }
        for (let i = 0; i < obj_data.length; i++) {
            const obj_data_item = obj_data[i];
            const path_data_item = path_data + '[' + i + ']';
            if (typeof obj_data_item !== 'object' || ArrayIsArray(obj_data_item) || obj_data_item === null) {
                return new TypeError('Expected "object" but received "' + typeof obj_data_item + '" (at "' + path_data_item + '")');
            }
        }
        const obj_done = obj.done;
        const path_done = path + '.done';
        if (typeof obj_done !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_done + '" (at "' + path_done + '")');
        }
        const obj_endTime = obj.endTime;
        const path_endTime = path + '.endTime';
        if (typeof obj_endTime !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_endTime + '" (at "' + path_endTime + '")');
        }
        const obj_metadata = obj.metadata;
        const path_metadata = path + '.metadata';
        if (typeof obj_metadata !== 'object' || ArrayIsArray(obj_metadata) || obj_metadata === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_metadata + '" (at "' + path_metadata + '")');
        }
        const obj_metadata_keys = ObjectKeys(obj_metadata);
        for (let i = 0; i < obj_metadata_keys.length; i++) {
            const key = obj_metadata_keys[i];
            const obj_metadata_prop = obj_metadata[key];
            const path_metadata_prop = path_metadata + '["' + key + '"]';
            if (typeof obj_metadata_prop !== 'object' || ArrayIsArray(obj_metadata_prop) || obj_metadata_prop === null) {
                return new TypeError('Expected "object" but received "' + typeof obj_metadata_prop + '" (at "' + path_metadata_prop + '")');
            }
        }
        const obj_queryId = obj.queryId;
        const path_queryId = path + '.queryId';
        if (typeof obj_queryId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_queryId + '" (at "' + path_queryId + '")');
        }
        const obj_rowCount = obj.rowCount;
        const path_rowCount = path + '.rowCount';
        if (typeof obj_rowCount !== 'number' || (typeof obj_rowCount === 'number' && Math.floor(obj_rowCount) !== obj_rowCount)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_rowCount + '" (at "' + path_rowCount + '")');
        }
        const obj_startTime = obj.startTime;
        const path_startTime = path + '.startTime';
        if (typeof obj_startTime !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_startTime + '" (at "' + path_startTime + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$1 = 'CdpQueryOutputRepresentation';
function keyBuilder$2(luvio, config) {
    return keyPrefix + '::' + RepresentationType$1 + ':' + config.id;
}
function keyBuilderFromType(luvio, object) {
    const keyParams = {
        id: object.queryId
    };
    return keyBuilder$2(luvio, keyParams);
}
function normalize$1(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$3 = function CdpQueryOutputRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1,
        private: [],
        opaque: true
    };
};
function equals$1(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1 = function CdpQueryOutputRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType(luvio, input);
    const ttlToUse = TTL;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$1, "cdp-query", VERSION$1, RepresentationType$1, equals$1);
    return createLink(key);
};
function getTypeCacheKeys$1(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$1,
        mergeable: false
    });
}

function select$2(luvio, params) {
    return select$3();
}
function getResponseCacheKeys$1(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1(storeKeyMap, luvio, response);
}
function ingestSuccess$1(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType(luvio, body);
    luvio.storeIngest(key, ingest$1, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2(),
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function createResourceRequest$1(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v64.0',
        basePath: '/ssot/query',
        method: 'post',
        body: config.body,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$1 = 'cdpQuery';
const cdpQuery_ConfigPropertyMetadata = [
    generateParamConfigMetadata('batchSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('dataspace', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('offset', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('orderby', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sql', true, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('userAgent', true, 2 /* Body */, 0 /* String */),
];
const cdpQuery_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$1, cdpQuery_ConfigPropertyMetadata);
const createResourceParams$1 = /*#__PURE__*/ createResourceParams$2(cdpQuery_ConfigPropertyMetadata);
function typeCheckConfig$1(untrustedConfig) {
    const config = {};
    typeCheckConfig$2(untrustedConfig, config, cdpQuery_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$1(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$1(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshot$1(luvio, config, options) {
    const resourceParams = createResourceParams$1(config);
    const request = createResourceRequest$1(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$1(luvio, resourceParams, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$1(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}
const cdpQueryAdapterFactory = (luvio) => {
    return function cdpQuery(untrustedConfig) {
        const config = validateAdapterConfig$1(untrustedConfig, cdpQuery_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            throw new Error('Invalid config for "cdpQuery"');
        }
        return buildNetworkSnapshot$1(luvio, config);
    };
};

const VERSION = "7df04f81d9b64d7cdebf01b86018ba25";
function validate(obj, path = 'CdpQueryDataOutputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_data = obj.data;
        const path_data = path + '.data';
        if (!ArrayIsArray(obj_data)) {
            return new TypeError('Expected "array" but received "' + typeof obj_data + '" (at "' + path_data + '")');
        }
        for (let i = 0; i < obj_data.length; i++) {
            const obj_data_item = obj_data[i];
            const path_data_item = path_data + '[' + i + ']';
            if (obj_data_item === undefined) {
                return new TypeError('Expected "defined" but received "' + typeof obj_data_item + '" (at "' + path_data_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType = 'CdpQueryDataOutputRepresentation';
function normalize(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1 = function CdpQueryDataOutputRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION,
        private: [],
        opaque: true
    };
};
function equals(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest = function CdpQueryDataOutputRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = path.ttl !== undefined ? path.ttl : 360000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize, "cdp-query", VERSION, RepresentationType, equals);
    return createLink(key);
};
function getTypeCacheKeys(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType,
        mergeable: false
    });
}

function select(luvio, params) {
    return select$1();
}
function keyBuilder$1(luvio, params) {
    return keyPrefix + '::CdpQueryDataOutputRepresentation:(' + 'dataspace:' + params.queryParams.dataspace + ',' + 'dataSourceId:' + params.urlParams.dataSourceId + ',' + 'dataSourceObjectId:' + params.urlParams.dataSourceObjectId + ',' + 'entityName:' + params.urlParams.entityName + ',' + 'sourceRecordId:' + params.urlParams.sourceRecordId + ')';
}
function getResponseCacheKeys(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys(storeKeyMap, luvio, response, () => keyBuilder$1(luvio, resourceParams));
}
function ingestSuccess(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1(luvio, resourceParams);
    luvio.storeIngest(key, ingest, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v64.0',
        basePath: '/ssot/universalIdLookup/' + config.urlParams.entityName + '/' + config.urlParams.dataSourceId + '/' + config.urlParams.dataSourceObjectId + '/' + config.urlParams.sourceRecordId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName = 'universalIdLookupBySourceId';
const universalIdLookupBySourceId_ConfigPropertyMetadata = [
    generateParamConfigMetadata('dataSourceId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('dataSourceObjectId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('entityName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('sourceRecordId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('dataspace', false, 1 /* QueryParameter */, 0 /* String */),
];
const universalIdLookupBySourceId_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName, universalIdLookupBySourceId_ConfigPropertyMetadata);
const createResourceParams = /*#__PURE__*/ createResourceParams$2(universalIdLookupBySourceId_ConfigPropertyMetadata);
function keyBuilder(luvio, config) {
    const resourceParams = createResourceParams(config);
    return keyBuilder$1(luvio, resourceParams);
}
function typeCheckConfig(untrustedConfig) {
    const config = {};
    typeCheckConfig$2(untrustedConfig, config, universalIdLookupBySourceId_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment(luvio, config) {
    createResourceParams(config);
    return select();
}
function onFetchResponseSuccess(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError(luvio, config, resourceParams, response) {
    const snapshot = ingestError(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot(luvio, config, options) {
    const resourceParams = createResourceParams(config);
    const request = createResourceRequest(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$1(context, coercedAdapterRequestContext, buildNetworkSnapshot, undefined, false);
}
function buildCachedSnapshotCachePolicy(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder(luvio, config),
        node: adapterFragment(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const universalIdLookupBySourceIdAdapterFactory = (luvio) => function cdpQuery__universalIdLookupBySourceId(untrustedConfig, requestContext) {
    const config = validateAdapterConfig(untrustedConfig, universalIdLookupBySourceId_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy, buildNetworkSnapshotCachePolicy);
};

let cdpQuery;
let universalIdLookupBySourceId;
// Imperative GET Adapters
let universalIdLookupBySourceId_imperative;
const universalIdLookupBySourceIdMetadata = {
    apiFamily: 'cdpquery',
    name: 'universalIdLookupBySourceId',
};
// Notify Update Available
function bindExportsTo(luvio) {
    // LDS Adapters
    const universalIdLookupBySourceId_ldsAdapter = createInstrumentedAdapter(createLDSAdapter(luvio, 'universalIdLookupBySourceId', universalIdLookupBySourceIdAdapterFactory), universalIdLookupBySourceIdMetadata);
    function unwrapSnapshotData(factory) {
        const adapter = factory(luvio);
        return (config) => adapter(config).then((snapshot) => snapshot.data);
    }
    return {
        cdpQuery: unwrapSnapshotData(cdpQueryAdapterFactory),
        universalIdLookupBySourceId: createWireAdapterConstructor(luvio, universalIdLookupBySourceId_ldsAdapter, universalIdLookupBySourceIdMetadata),
        // Imperative GET Adapters
        universalIdLookupBySourceId_imperative: createImperativeAdapter(luvio, universalIdLookupBySourceId_ldsAdapter, universalIdLookupBySourceIdMetadata),
        // Notify Update Availables
    };
}
withDefaultLuvio((luvio) => {
    ({ cdpQuery, universalIdLookupBySourceId, universalIdLookupBySourceId_imperative } =
        bindExportsTo(luvio));
});

export { cdpQuery, universalIdLookupBySourceId, universalIdLookupBySourceId_imperative };
// version: 1.340.0-08235a5e1b
